@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2016, Brandon Nielsen
+# Copyright (c) 2018, Brandon Nielsen
 # All rights reserved.
 #
 # This software may be modified and distributed under the terms
@@ -10,6 +10,7 @@ import datetime
 import unittest
 import dateutil
 
+from aniso8601.exceptions import ISOFormatError, RelativeValueError
 from aniso8601.duration import parse_duration, _parse_duration_prescribed, \
         _parse_duration_combined, _parse_duration_prescribed_notime, \
         _parse_duration_prescribed_time, _parse_duration_element, \
@@ -41,6 +42,14 @@ class TestDurationParserFunctions(unittest.TestCase):
         resultduration = parse_duration('PT4H54M6,5S')
         self.assertEqual(resultduration, datetime.timedelta(hours=4, minutes=54, seconds=6.5))
 
+        #Make sure we truncate, not round
+        #https://bitbucket.org/nielsenb/aniso8601/issues/10/sub-microsecond-precision-in-durations-is
+        resultduration = parse_duration('PT0.0000001S')
+        self.assertEqual(resultduration, datetime.timedelta(0))
+
+        resultduration = parse_duration('PT2.0000048S')
+        self.assertEqual(resultduration, datetime.timedelta(seconds=2, microseconds=4))
+
         resultduration = parse_duration('P1Y')
         self.assertEqual(resultduration, datetime.timedelta(days=365))
 
@@ -83,16 +92,21 @@ class TestDurationParserFunctions(unittest.TestCase):
         resultduration = parse_duration('P0003-06-04T12:30:05.5')
         self.assertEqual(resultduration, datetime.timedelta(days=1279, hours=12, minutes=30, seconds=5.5))
 
+        #Make sure we truncate, not round
+        #https://bitbucket.org/nielsenb/aniso8601/issues/10/sub-microsecond-precision-in-durations-is
+        resultduration = parse_duration('P0001-02-03T14:43:59.9999997')
+        self.assertEqual(resultduration, datetime.timedelta(days=428, hours=14, minutes=43, seconds=59, microseconds=999999))
+
         #Verify overflows
         self.assertEqual(parse_duration('PT36H'), parse_duration('P1DT12H'))
 
     def test_parse_duration_nop(self):
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             #Duration must start with a P
             parse_duration('1Y2M3DT4H54M6S')
 
     def test_parse_duration_weekcombination(self):
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             #Week designator cannot be combined with other time designators
             #https://bitbucket.org/nielsenb/aniso8601/issues/2/week-designators-should-not-be-combinable
             parse_duration('P1Y2W')
@@ -101,31 +115,31 @@ class TestDurationParserFunctions(unittest.TestCase):
         #Ensure durations are required to be in the correct order
         #https://bitbucket.org/nielsenb/aniso8601/issues/7/durations-with-time-components-before-t
         #https://bitbucket.org/nielsenb/aniso8601/issues/8/durations-with-components-in-wrong-order
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             parse_duration('P1S')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             parse_duration('P1D1S')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             parse_duration('P1H1M')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             parse_duration('1Y2M3D1SPT1M')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             parse_duration('P1Y2M3D2MT1S')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             parse_duration('P2M3D1ST1Y1M')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             parse_duration('P1Y2M2MT3D1S')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             parse_duration('P1D1Y1M')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             parse_duration('PT1S1H')
 
     def test_parse_duration_suffixgarbage(self):
@@ -202,19 +216,19 @@ class TestDurationParserFunctions(unittest.TestCase):
         self.assertEqual(parse_duration('PT36H'), parse_duration('P1DT12H'))
 
     def test_parse_duration_prescribed_multiplefractions(self):
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             #Multiple fractions are not allowed
             _parse_duration_prescribed('P1Y2M3DT4H5.1234M6.1234S', False)
 
     def test_parse_duration_prescribed_middlefraction(self):
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             #Fraction only allowed on final component
             _parse_duration_prescribed('P1Y2M3DT4H5.1234M6S', False)
 
     def test_parse_duration_prescribed_suffixgarbage(self):
         #Don't allow garbage after the duration
         #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed('P1Dasdfasdf', False)
 
     def test_parse_duration_prescribed_notime(self):
@@ -265,28 +279,28 @@ class TestDurationParserFunctions(unittest.TestCase):
 
     def test_parse_duration_prescribed_notime_timepart(self):
         #Ensure no time part is allowed
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_notime('P1S')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_notime('P1D1S')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_notime('P1H1M')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_notime('P1Y2M3D4H')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_notime('P1Y2M3D4H5S')
 
     def test_parse_duration_prescribed_notime_outoforder(self):
         #Ensure durations are required to be in the correct order
         #https://bitbucket.org/nielsenb/aniso8601/issues/8/durations-with-components-in-wrong-order
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_notime('P1H1M')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_notime('P1D1Y1M')
 
     def test_parse_duration_prescribed_time(self):
@@ -307,33 +321,33 @@ class TestDurationParserFunctions(unittest.TestCase):
 
     def test_parse_duration_prescribed_time_timeindate(self):
         #Don't allow time components in date half
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_time('P1Y2M3D4HT54M6S')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_time('P1Y2M3D6ST4H54M')
 
     def test_parse_duration_prescribed_time_dateintime(self):
         #Don't allow date components in time half
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_time('P2M3DT1Y4H54M6S')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_time('P1Y2MT3D4H54M6S')
 
     def test_parse_duration_prescribed_time_outoforder(self):
         #Ensure durations are required to be in the correct order
         #https://bitbucket.org/nielsenb/aniso8601/issues/7/durations-with-time-components-before-t
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_time('1Y2M3D1SPT1M')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_time('P1Y2M3D2MT1S')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_time('P2M3D1ST1Y1M')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed_time('P1Y2M2MT3D1S')
 
     def test_parse_duration_combined(self):
@@ -378,6 +392,11 @@ class TestRelativeDurationParserFunctions(unittest.TestCase):
         resultduration = parse_duration('P0003-06-04T12:30:05.5', relative=True)
         self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(years=3, months=6, days=4, hours=12, minutes=30, seconds=5, microseconds=500000))
 
+        #Make sure we truncate, not round
+        #https://bitbucket.org/nielsenb/aniso8601/issues/10/sub-microsecond-precision-in-durations-is
+        resultduration = parse_duration('P0001-02-03T14:43:59.9999997', relative=True)
+        self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(years=1, months=2, days=3, hours=14, minutes=43, seconds=59, microseconds=999999))
+
     def test_parse_duration_prescribed_relative(self):
         resultduration = _parse_duration_prescribed('P1Y', True)
         self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(years=1))
@@ -393,32 +412,40 @@ class TestRelativeDurationParserFunctions(unittest.TestCase):
         resultduration = _parse_duration_prescribed('P1.5W', True)
         self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(days=10.5))
 
+        #Make sure we truncate, not round
+        #https://bitbucket.org/nielsenb/aniso8601/issues/10/sub-microsecond-precision-in-durations-is
+        resultduration = parse_duration('PT0.0000001S', relative=True)
+        self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(0))
+
+        resultduration = parse_duration('PT2.0000048S', relative=True)
+        self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(seconds=2.000004))
+
     def test_parse_duration_prescribed_relative_multiplefractions(self):
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             #Multiple fractions are not allowed
             _parse_duration_prescribed('P1Y2M3DT4H5.1234M6.1234S', True)
 
     def test_parse_duration_prescribed_relative_middlefraction(self):
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             #Fraction only allowed on final component
             _parse_duration_prescribed('P1Y2M3DT4H5.1234M6S', True)
 
     def test_parse_duration_prescribed_relative_suffixgarbage(self):
         #Don't allow garbage after the duration
         #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed('P1Dasdfasdf', True)
 
     def test_parse_duration_prescribed_relative_fractionalyear(self):
         #Fractional months and years are not defined
         #https://github.com/dateutil/dateutil/issues/40
-        with self.assertRaises(ValueError):
+        with self.assertRaises(RelativeValueError):
             _parse_duration_prescribed('P1.5Y', True)
 
     def test_parse_duration_prescribed_relative_fractionalmonth(self):
         #Fractional months and years are not defined
         #https://github.com/dateutil/dateutil/issues/40
-        with self.assertRaises(ValueError):
+        with self.assertRaises(RelativeValueError):
             _parse_duration_prescribed('P1.5M', True)
 
     def test_parse_duration_prescribed_relative_nodateutil(self):
@@ -439,31 +466,31 @@ class TestRelativeDurationParserFunctions(unittest.TestCase):
         #Ensure durations are required to be in the correct order
         #https://bitbucket.org/nielsenb/aniso8601/issues/7/durations-with-time-components-before-t
         #https://bitbucket.org/nielsenb/aniso8601/issues/8/durations-with-components-in-wrong-order
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed('P1S', True)
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed('P1D1S', True)
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed('P1H1M', True)
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed('1Y2M3D1SPT1M', True)
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed('P1Y2M3D2MT1S', True)
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed('P2M3D1ST1Y1M', True)
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed('P1Y2M2MT3D1S', True)
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed('P1D1Y1M', True)
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             _parse_duration_prescribed('PT1S1H', True)
 
     def test_parse_duration_combined_relative(self):
