@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2016, Brandon Nielsen
+# Copyright (c) 2018, Brandon Nielsen
 # All rights reserved.
 #
 # This software may be modified and distributed under the terms
@@ -11,6 +11,7 @@ import datetime
 import dateutil.relativedelta
 
 from aniso8601 import compat
+from aniso8601.exceptions import ISOFormatError, RelativeValueError
 from aniso8601.interval import parse_interval, parse_repeating_interval, _parse_interval_parts
 
 class TestIntervalParserFunctions(unittest.TestCase):
@@ -23,6 +24,10 @@ class TestIntervalParserFunctions(unittest.TestCase):
         self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
         self.assertEqual(resultinterval[1], datetime.date(year=1981, month=3, day=6))
 
+        resultinterval = parse_interval('P1.5Y/2018-03-06')
+        self.assertEqual(resultinterval[0], datetime.date(year=2018, month=3, day=6))
+        self.assertEqual(resultinterval[1], datetime.date(year=2016, month=9, day=5))
+
         resultinterval = parse_interval('PT1H/2014-11-12')
         self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
         self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=11, hour=23))
@@ -31,6 +36,16 @@ class TestIntervalParserFunctions(unittest.TestCase):
         self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
         self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=11, hour=19, minute=5, second=53, microsecond=500000))
 
+        #Make sure we truncate, not round
+        #https://bitbucket.org/nielsenb/aniso8601/issues/10/sub-microsecond-precision-in-durations-is
+        resultinterval = parse_interval('PT0.0000001S/2018-03-06')
+        self.assertEqual(resultinterval[0], datetime.date(year=2018, month=3, day=6))
+        self.assertEqual(resultinterval[1], datetime.datetime(year=2018, month=3, day=6))
+
+        resultinterval = parse_interval('PT2.0000048S/2018-03-06')
+        self.assertEqual(resultinterval[0], datetime.date(year=2018, month=3, day=6))
+        self.assertEqual(resultinterval[1], datetime.datetime(year=2018, month=3, day=5, hour=23, minute=59, second=57, microsecond=999996))
+
         resultinterval = parse_interval('1981-04-05T01:01:00/P1M1DT1M')
         self.assertEqual(resultinterval[0], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
         self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=5, day=6, hour=1, minute=2))
@@ -39,6 +54,10 @@ class TestIntervalParserFunctions(unittest.TestCase):
         self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
         self.assertEqual(resultinterval[1], datetime.date(year=1981, month=5, day=6))
 
+        resultinterval = parse_interval('2018-03-06/P2.5M')
+        self.assertEqual(resultinterval[0], datetime.date(year=2018, month=3, day=6))
+        self.assertEqual(resultinterval[1], datetime.date(year=2018, month=5, day=20))
+
         resultinterval = parse_interval('2014-11-12/PT1H')
         self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
         self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=12, hour=1, minute=0))
@@ -47,6 +66,16 @@ class TestIntervalParserFunctions(unittest.TestCase):
         self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
         self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=12, hour=4, minute=54, second=6, microsecond=500000))
 
+        #Make sure we truncate, not round
+        #https://bitbucket.org/nielsenb/aniso8601/issues/10/sub-microsecond-precision-in-durations-is
+        resultinterval = parse_interval('2018-03-06/PT0.0000001S')
+        self.assertEqual(resultinterval[0], datetime.date(year=2018, month=3, day=6))
+        self.assertEqual(resultinterval[1], datetime.datetime(year=2018, month=3, day=6))
+
+        resultinterval = parse_interval('2018-03-06/PT2.0000048S')
+        self.assertEqual(resultinterval[0], datetime.date(year=2018, month=3, day=6))
+        self.assertEqual(resultinterval[1], datetime.datetime(year=2018, month=3, day=6, hour=0, minute=0, second=2, microsecond=4))
+
         resultinterval = parse_interval('1980-03-05T01:01:00/1981-04-05T01:01:00')
         self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
         self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
@@ -67,6 +96,13 @@ class TestIntervalParserFunctions(unittest.TestCase):
         self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
         self.assertEqual(resultinterval[1], datetime.date(year=1980, month=3, day=5))
 
+        #Make sure we truncate, not round
+        #https://bitbucket.org/nielsenb/aniso8601/issues/10/sub-microsecond-precision-in-durations-is
+        resultinterval = parse_interval('1980-03-05T01:01:00.0000001/1981-04-05T14:43:59.9999997')
+        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
+        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=14, minute=43, second=59, microsecond=999999))
+
+        #Test different separators
         resultinterval = parse_interval('1980-03-05T01:01:00--1981-04-05T01:01:00', intervaldelimiter='--')
         self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
         self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
@@ -75,6 +111,17 @@ class TestIntervalParserFunctions(unittest.TestCase):
         self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
         self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
 
+    def test_parse_interval_repeating(self):
+        #Parse interval can't parse repeating intervals
+        with self.assertRaises(ISOFormatError):
+            parse_interval('R3/1981-04-05/P1D')
+
+        with self.assertRaises(ISOFormatError):
+            parse_interval('R3/1981-04-05/P0003-06-04T12:30:05.5')
+
+        with self.assertRaises(ISOFormatError):
+            parse_interval('R/PT1H2M/1980-03-05T01:01:00')
+
     def test_parse_interval_suffixgarbage(self):
         #Don't allow garbage after the duration
         #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
@@ -108,6 +155,16 @@ class TestRelativeIntervalParserFunctions(unittest.TestCase):
         self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
         self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=11, hour=19, minute=5, second=53, microsecond=500000))
 
+        #Make sure we truncate, not round
+        #https://bitbucket.org/nielsenb/aniso8601/issues/10/sub-microsecond-precision-in-durations-is
+        resultinterval = parse_interval('PT0.0000001S/2018-03-06', relative=True)
+        self.assertEqual(resultinterval[0], datetime.date(year=2018, month=3, day=6))
+        self.assertEqual(resultinterval[1], datetime.datetime(year=2018, month=3, day=6))
+
+        resultinterval = parse_interval('PT2.0000048S/2018-03-06', relative=True)
+        self.assertEqual(resultinterval[0], datetime.date(year=2018, month=3, day=6))
+        self.assertEqual(resultinterval[1], datetime.datetime(year=2018, month=3, day=5, hour=23, minute=59, second=57, microsecond=999996))
+
         resultinterval = parse_interval('1981-04-05T01:01:00/P1M1DT1M', relative=True)
         self.assertEqual(resultinterval[0], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
         self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=5, day=6, hour=1, minute=2))
@@ -158,6 +215,16 @@ class TestRelativeIntervalParserFunctions(unittest.TestCase):
         self.assertEqual(resultinterval[0], datetime.date(year=2001, month=3, day=1))
         self.assertEqual(resultinterval[1], datetime.date(year=2000, month=3, day=1))
 
+        #Make sure we truncate, not round
+        #https://bitbucket.org/nielsenb/aniso8601/issues/10/sub-microsecond-precision-in-durations-is
+        resultinterval = parse_interval('2018-03-06/PT0.0000001S', relative=True)
+        self.assertEqual(resultinterval[0], datetime.date(year=2018, month=3, day=6))
+        self.assertEqual(resultinterval[1], datetime.datetime(year=2018, month=3, day=6))
+
+        resultinterval = parse_interval('2018-03-06/PT2.0000048S', relative=True)
+        self.assertEqual(resultinterval[0], datetime.date(year=2018, month=3, day=6))
+        self.assertEqual(resultinterval[1], datetime.datetime(year=2018, month=3, day=6, hour=0, minute=0, second=2, microsecond=4))
+
         resultinterval = parse_interval('1980-03-05T01:01:00/1981-04-05T01:01:00', relative=True)
         self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
         self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
@@ -178,6 +245,13 @@ class TestRelativeIntervalParserFunctions(unittest.TestCase):
         self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
         self.assertEqual(resultinterval[1], datetime.date(year=1980, month=3, day=5))
 
+        #Make sure we truncate, not round
+        #https://bitbucket.org/nielsenb/aniso8601/issues/10/sub-microsecond-precision-in-durations-is
+        resultinterval = parse_interval('1980-03-05T01:01:00.0000001/1981-04-05T14:43:59.9999997', relative=True)
+        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
+        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=14, minute=43, second=59, microsecond=999999))
+
+        #Test different separators
         resultinterval = parse_interval('1980-03-05T01:01:00--1981-04-05T01:01:00', intervaldelimiter='--', relative=True)
         self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
         self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
@@ -186,6 +260,18 @@ class TestRelativeIntervalParserFunctions(unittest.TestCase):
         self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
         self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
 
+    def test_parse_interval_relative_fractionalyear(self):
+        #Fractional months and years are not defined
+        #https://github.com/dateutil/dateutil/issues/40
+        with self.assertRaises(RelativeValueError):
+            parse_interval('P1.5Y/2018-03-06', relative=True)
+
+    def test_parse_interval_relative_fractionalmonth(self):
+        #Fractional months and years are not defined
+        #https://github.com/dateutil/dateutil/issues/40
+        with self.assertRaises(RelativeValueError):
+            parse_interval('2018-03-06/P2.5M', relative=True)
+
     def test_parse_interval_relative_suffixgarbage(self):
         #Don't allow garbage after the duration
         #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
@@ -230,10 +316,10 @@ class TestRepeatingIntervalParserFunctions(unittest.TestCase):
         #Don't allow garbage after the duration
         #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
         with self.assertRaises(ValueError):
-            parse_interval('R3/1981-04-05/P1Dasdf')
+            parse_repeating_interval('R3/1981-04-05/P1Dasdf')
 
         with self.assertRaises(ValueError):
-            parse_interval('R3/1981-04-05/P0003-06-04T12:30:05.5asdfasdf')
+            parse_repeating_interval('R3/1981-04-05/P0003-06-04T12:30:05.5asdfasdf')
 
     def test_parse_repeating_interval_relative(self):
         results = list(parse_repeating_interval('R3/1981-04-05/P1D', relative=True))
@@ -270,10 +356,10 @@ class TestRepeatingIntervalParserFunctions(unittest.TestCase):
     def test_parse_repeating_interval_relative_suffixgarbage(self):
         #Don't allow garbage after the duration
         #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             parse_interval('R3/1981-04-05/P1Dasdf', relative=True)
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ISOFormatError):
             parse_interval('R3/1981-04-05/P0003-06-04T12:30:05.5asdfasdf')
 
     def test_parse_repeating_interval_relative_nodateutil(self):
