@@ -1,18 +1,20 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2016, Brandon Nielsen
+# Copyright (c) 2018, Brandon Nielsen
 # All rights reserved.
 #
 # This software may be modified and distributed under the terms
 # of the BSD license.  See the LICENSE file for details.
 
-import unittest
 import datetime
+import unittest
 
+from aniso8601.exceptions import HoursOutOfBoundsError, LeapSecondError, \
+        MidnightBoundsError, MinutesOutOfBoundsError, SecondsOutOfBoundsError
 from aniso8601.resolution import TimeResolution
-from aniso8601.time import get_time_resolution, parse_time, parse_datetime, \
-     _parse_time_naive, _parse_hour, _parse_minute_time, _parse_second_time, \
-     _build_time, _split_tz
+from aniso8601.time import get_time_resolution, parse_datetime, parse_time, \
+    _parse_time_naive, _parse_hour, _parse_minute_time, _parse_second_time, \
+    _build_time, _split_tz
 
 class TestTimeParserFunctions(unittest.TestCase):
     def test_get_time_resolution(self):
@@ -64,6 +66,9 @@ class TestTimeParserFunctions(unittest.TestCase):
         time = parse_time('23:21:28.512400')
         self.assertEqual(time, datetime.time(hour=23, minute=21, second=28, microsecond=512400))
 
+        time = parse_time('14:43:59.9999997')
+        self.assertEqual(time, datetime.time(hour=14, minute=43, second=59, microsecond=999999))
+
         time = parse_time('01:23')
         self.assertEqual(time, datetime.time(hour=1, minute=23))
 
@@ -79,6 +84,9 @@ class TestTimeParserFunctions(unittest.TestCase):
         time = parse_time('240000')
         self.assertEqual(time, datetime.time(hour=0))
 
+        time = parse_time('144359.9999997')
+        self.assertEqual(time, datetime.time(hour=14, minute=43, second=59, microsecond=999999))
+
         time = parse_time('0123')
         self.assertEqual(time, datetime.time(hour=1, minute=23))
 
@@ -185,22 +193,47 @@ class TestTimeParserFunctions(unittest.TestCase):
         self.assertEqual(tzinfoobject.utcoffset(None), datetime.timedelta(hours=0))
         self.assertEqual(tzinfoobject.tzname(None), 'UTC')
 
-        with self.assertRaises(ValueError):
+    def test_parse_time_bounds(self):
+        #Leap seconds not supported
+        #https://bitbucket.org/nielsenb/aniso8601/issues/10/sub-microsecond-precision-in-durations-is
+        #https://bitbucket.org/nielsenb/aniso8601/issues/13/parsing-of-leap-second-gives-wildly
+        with self.assertRaises(LeapSecondError):
+            parse_time('23:59:60')
+
+        with self.assertRaises(LeapSecondError):
+            parse_time('23:59:60Z')
+
+        with self.assertRaises(LeapSecondError):
+            parse_time('23:59:60+00:00')
+
+        #Leap seconds not supported, but exception type should be changed
+        with self.assertRaises(SecondsOutOfBoundsError):
             parse_time('00:00:60')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(SecondsOutOfBoundsError):
             parse_time('00:00:60Z')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(SecondsOutOfBoundsError):
             parse_time('00:00:60+00:00')
 
-        with self.assertRaises(ValueError):
+        #Seconds must not be greater than or equal to 60
+        with self.assertRaises(SecondsOutOfBoundsError):
+            parse_time('00:00:61')
+
+        with self.assertRaises(SecondsOutOfBoundsError):
+            parse_time('00:00:61Z')
+
+        with self.assertRaises(SecondsOutOfBoundsError):
+            parse_time('00:00:61+00:00')
+
+        #Minutes must not be greater than 60
+        with self.assertRaises(MinutesOutOfBoundsError):
             parse_time('00:61')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MinutesOutOfBoundsError):
             parse_time('00:61Z')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MinutesOutOfBoundsError):
             parse_time('00:61+00:00')
 
     def test_parse_datetime(self):
@@ -232,29 +265,52 @@ class TestTimeParserFunctions(unittest.TestCase):
         self.assertEqual(tzinfoobject.utcoffset(None), datetime.timedelta(hours=0))
         self.assertEqual(tzinfoobject.tzname(None), '+00:00')
 
+    def test_parse_datetime_bounds(self):
+        #Leap seconds not supported
+        #https://bitbucket.org/nielsenb/aniso8601/issues/10/sub-microsecond-precision-in-durations-is
         #https://bitbucket.org/nielsenb/aniso8601/issues/13/parsing-of-leap-second-gives-wildly
-        with self.assertRaises(ValueError):
+        with self.assertRaises(LeapSecondError):
             parse_datetime('2016-12-31T23:59:60+00:00')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(LeapSecondError):
             parse_datetime('2016-12-31T23:59:60')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(LeapSecondError):
+            parse_datetime('2016-12-31T23:59:60Z')
+
+        #Seconds can't be greater than 60
+        with self.assertRaises(SecondsOutOfBoundsError):
             parse_datetime('1981-04-05T00:00:60')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(SecondsOutOfBoundsError):
             parse_datetime('1981-04-05T00:00:60Z')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(SecondsOutOfBoundsError):
             parse_datetime('1981-04-05T00:00:60+00:00')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(SecondsOutOfBoundsError):
+            parse_datetime('2016-12-31T23:59:61+00:00')
+
+        with self.assertRaises(SecondsOutOfBoundsError):
+            parse_datetime('2016-12-31T23:59:61')
+
+        with self.assertRaises(SecondsOutOfBoundsError):
+            parse_datetime('1981-04-05T00:00:61')
+
+        with self.assertRaises(SecondsOutOfBoundsError):
+            parse_datetime('1981-04-05T00:00:61Z')
+
+        with self.assertRaises(SecondsOutOfBoundsError):
+            parse_datetime('1981-04-05T00:00:61+00:00')
+
+        #Minutes can't be greater than 60
+        with self.assertRaises(MinutesOutOfBoundsError):
             parse_datetime('1981-04-05T00:61')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MinutesOutOfBoundsError):
             parse_datetime('1981-04-05T00:61Z')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MinutesOutOfBoundsError):
             parse_datetime('1981-04-05T00:61+00:00')
 
     def test_parse_datetime_spaceseperated(self):
@@ -311,15 +367,30 @@ class TestTimeParserFunctions(unittest.TestCase):
         time = _parse_time_naive('01.4567')
         self.assertEqual(time, datetime.time(hour=1, minute=27, second=24, microsecond=120000))
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MinutesOutOfBoundsError):
+            _parse_time_naive('00:61')
+
+    def test_parse_time_naive_bounds(self):
+        #Leap seconds not supporter
+        with self.assertRaises(LeapSecondError):
+            _parse_time_naive('23:59:60')
+
+        #Seconds must not be greater than or equal to 60
+        with self.assertRaises(SecondsOutOfBoundsError):
             _parse_time_naive('00:00:60')
 
+        with self.assertRaises(SecondsOutOfBoundsError):
+            _parse_time_naive('00:00:61')
+
+        #Minutes must not be greater than 60
+        with self.assertRaises(MinutesOutOfBoundsError):
+            _parse_time_naive('00:61')
+
+    def test_parse_time_naive_tz(self):
+        #These fail because of the timezone being parsed by the naive parser
         with self.assertRaises(ValueError):
             _parse_time_naive('00:00:60+00:00')
 
-        with self.assertRaises(ValueError):
-            _parse_time_naive('00:61')
-
         with self.assertRaises(ValueError):
             _parse_time_naive('00:61+00:00')
 
@@ -336,7 +407,9 @@ class TestTimeParserFunctions(unittest.TestCase):
         time = _parse_hour('12.5')
         self.assertEqual(time, datetime.time(hour=12, minute=30))
 
-        with self.assertRaises(ValueError):
+    def test_parse_hour_bounds(self):
+        #Hour cannot be larger than 24
+        with self.assertRaises(HoursOutOfBoundsError):
             _parse_hour('24.1')
 
     def test_parse_minute_time(self):
@@ -358,28 +431,31 @@ class TestTimeParserFunctions(unittest.TestCase):
         time = _parse_minute_time('0123.4567')
         self.assertEqual(time, datetime.time(hour=1, minute=23, second=27, microsecond=402000))
 
-        with self.assertRaises(ValueError):
+    def test_parse_minute_time_bounds(self):
+        #Minutes cannot be greater than 60
+        with self.assertRaises(MinutesOutOfBoundsError):
             _parse_minute_time('0060')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MinutesOutOfBoundsError):
             _parse_minute_time('0060.1')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MinutesOutOfBoundsError):
             _parse_minute_time('00:60')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MinutesOutOfBoundsError):
             _parse_minute_time('00:60.1')
 
-        with self.assertRaises(ValueError):
+        #Hour 24 can only represent midnight
+        with self.assertRaises(MidnightBoundsError):
             _parse_minute_time('2401')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MidnightBoundsError):
             _parse_minute_time('2400.1')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MidnightBoundsError):
             _parse_minute_time('24:01')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MidnightBoundsError):
             _parse_minute_time('24:00.1')
 
     def test_parse_second_time(self):
@@ -392,6 +468,9 @@ class TestTimeParserFunctions(unittest.TestCase):
         time = _parse_second_time('23:21:28.512400')
         self.assertEqual(time, datetime.time(hour=23, minute=21, second=28, microsecond=512400))
 
+        time = _parse_second_time('14:43:59.9999997')
+        self.assertEqual(time, datetime.time(hour=14, minute=43, second=59, microsecond=999999))
+
         time = _parse_second_time('012345')
         self.assertEqual(time, datetime.time(hour=1, minute=23, second=45))
 
@@ -401,28 +480,49 @@ class TestTimeParserFunctions(unittest.TestCase):
         time = _parse_second_time('232128.512400')
         self.assertEqual(time, datetime.time(hour=23, minute=21, second=28, microsecond=512400))
 
-        with self.assertRaises(ValueError):
+        time = _parse_second_time('144359.9999997')
+        self.assertEqual(time, datetime.time(hour=14, minute=43, second=59, microsecond=999999))
+
+    def test_parse_second_time_bounds(self):
+        #Leap seconds not supported
+        with self.assertRaises(LeapSecondError):
+            _parse_second_time('235960')
+
+        with self.assertRaises(LeapSecondError):
+            _parse_second_time('23:59:60')
+
+        #Seconds must be less than 60
+        #Leap seconds not supported
+        with self.assertRaises(SecondsOutOfBoundsError):
             _parse_second_time('000060')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(SecondsOutOfBoundsError):
             _parse_second_time('00:00:60')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(SecondsOutOfBoundsError):
+            _parse_second_time('000061')
+
+        with self.assertRaises(SecondsOutOfBoundsError):
+            _parse_second_time('00:00:61')
+
+        #Minutes must be less than 60
+        with self.assertRaises(MinutesOutOfBoundsError):
             _parse_second_time('006000')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MinutesOutOfBoundsError):
             _parse_second_time('00:60:00')
 
-        with self.assertRaises(ValueError):
+        #Hour 24 can only represent midnight
+        with self.assertRaises(MidnightBoundsError):
             _parse_second_time('240001')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MidnightBoundsError):
             _parse_second_time('24:00:01')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MidnightBoundsError):
             _parse_second_time('240100')
 
-        with self.assertRaises(ValueError):
+        with self.assertRaises(MidnightBoundsError):
             _parse_second_time('24:01:00')
 
     def test_build_time(self):
